% HUANG-KECMAN-KOPRIVA BOOK
% CHAPTER 6
% MATLAB CODES FOR THE UNSUPERVISED LEARNING ALGORITHMS

%**************************************************************************
% 
% Displays scatter plots of two uniformly distributed source signals,
% their mixture, PCA transformed signals and ICA transformed signals shown 
% in Figures 1, 2, 3 and 4 respectively.
% Corresponding Figures in the book are Figure 6.16 right, 6.17, 6.18 and
% 6.25.
% 
%*************************************************************************

clear all;	close all;	format compact

T = 10000;                              % sample size

s1 = rand(1,T);                         % uniformly distributed source signal s1
s2 = rand(1,T);                         % uniformly distributed source signal s2
s1 = s1 - mean(s1);                     % make source 1 zero mean
s2 = s2 - mean(s2);                     % make source 2 zero mean

figure(1)                                  % Figure 6.16 right
plot(s1,s2,'ko')
grid
title('Source signals')
text(0.55,-0.6,'s1')
xlabel('Figure 6.16 Scatter plot of two uniformly distributed source signals: 10,000 samples')
ylabel('s2')
axis([-0.6 0.6 -0.6 0.6])
display('Press any key to continue ...')
pause

S = [s1; s2];                          % matrix of source signals
A = [2 1;1 1];                         % mixing matrix Eq.(6.2)
X=A*S;                                 % matrix of mixed signals

figure(2)                                 % scatter plot of mixed signals (Fig. 6.17)
plot(X(1,:),X(2,:),'ko')
grid
title('Mixed signals')
text(1.25,-1.05,'x1')
xlabel('Figure 6.17 Scatter plot of 2 mixed signals obtained by linear mixing of 2 uniformly distributed source signals')
ylabel('x2')
axis([-1.5 1.5 -1.0 1.0])
display('Press any key to continue ...')
pause

% PCA transform
Rx = cov(X');                        % data covariance matrix
[E,LAM] = eig(Rx);                   % eigen-decomposition of Rx
eigv = diag(LAM);
[eigvsort,ind] = sort(eigv);
E = E(:,ind);
Z = E'*X;                            % the PCA transform

figure(3)                               % scatter plot of PCA recovered source signals (Fig. 6.18)
plot(Z(1,:),Z(2,:),'ko')
grid
title('PCA transformed signals')
xlabel('Figure 6.18 Scatter plot of two PCA transformed signals')
text(0.44,-2.2,'z1')
ylabel('z2')
axis([-0.5 0.5 -2.0 2.0])
display('Press any key to continue ...')
pause

% ICA transform by the minimum of the mutual information
gain = 0.1;                         % learning gain
theta = 3;                          % Gaussian exponent for sub-Gaussian signals
ITMAX = 400;                        % maximal number of iterations
it = 0;                             % iteration counter
I = eye(2);                         % identity matrix
W = I;                              % initial value for de-mixing matrix
h = waitbar(0,'ICA transform. Please wait...');
while (it < ITMAX)                  % iterate until it > ITMAX
  it = it +1;
  Y = W*Z;                          % ICA transform Eq. (6.3)
  for i = 1:2
      fi(i,:) = sign(Y(i,:)).*power(abs(Y(i,:)),theta-1); % score function Eq.(6.48)
  end
  F = I - fi*Y'/T;                     % Eq. (6.42)
  W = W + gain*F*W;                    % Eq. (6.42)
  waitbar(it/ITMAX,h)
end
close (h)

figure(4)                               % scatter plot of ICA recovered source signals (Fig. 6.25)
plot(Y(1,:),Y(2,:),'ko')
grid
title('ICA transformed signals')
text(1.7,-2.1,'y1')
xlabel('Figure 6.25 Scatter plot of the signals recovered by the ICA algorithm (6.42)')
ylabel('y2')
axis([-2 2 -2.0 2.0])
