% HUANG-KECMAN-KOPRIVA BOOK
% CHAPTER 6
% MATLAB CODES FOR THE UNSUPERVISED LEARNING ALGORITHMS

%**************************************************************************
% 
% Displays scatter plots of two normally distributed source signals,
% their mixture and PCA transformed signals. Scatter plots are shown in 
% Figures 1, 2 and 3. Corresponding Figures in the book are 6.21, 
% 6.22 and 6.23 respectively.
%
%*************************************************************************

clear all;	close all;	format compact

T = 10000;                              % sample size

s1 = randn(1,T);                        % normally distributed source signal s1
s2 = randn(1,T);                        % normally distributed source signal s2

figure(1)                                  % Figure 6.21 
plot(s1,s2,'ko')
grid
title('Source signals')
text(4.4,-5.3,'s1')
xlabel('Figure 6.21 Scatter plot of two normally distributed source signals')
ylabel('s2')
axis([-4 5 -5 4])
display('Press any key to continue ...')

pause

S = [s1; s2];                          % matrix of source signals
A = [2 1;1 1];                         % mixing matrix Eq.(6.2)
X=A*S;                                 % matrix of mixed signals

figure(2)                                 % scatter plot of mixed signals (Fig. 6.22)
plot(X(1,:),X(2,:),'ko')
grid
title('Mixed signals')
text(8.8,-6.4,'x1')
xlabel('Figure 6.22 Scatter plot of 2 mixed signals obtained by linear mixing of 2 normally distributed source signals')
ylabel('x2')
axis([-10 10 -6 6])
display('Press any key to continue ...')

pause

% Whitening transform
Rx = cov(X');                        % data covariance matrix
[E,LAM] = eig(Rx);                   % eigen-decomposition of Rx
lam = diag(LAM);                      % extract eigen-values
[lam,ind] = sort(lam);
E = E(:,ind);
lam = sqrt(1./lam);                  % square root of inverse
LAM = diag(lam);                     % back into diagonal matrix form
Z = LAM*E'*X;                        % the whitening transform

figure(3)                               % scatter plot of PCA recovered source signals (Fig. 6.23)
plot(Z(1,:),Z(2,:),'ko')
grid
title('PCA transformed signals')
text(4.4,-5.3,'z1')
xlabel('Figure 6.23 Scatter plot of two signals obtained by whitening (PCA) transform')
ylabel('z2')
axis([-4 5 -5 4])

